@@ -1,6 +1,7 @@
 
 from datetime import datetime
 
+from django.contrib.contenttypes.generic import GenericForeignKey
 from django.contrib.sites.models import Site
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import models
@@ -8,6 +9,7 @@ from django.db.models.base import ModelBase
 from django.template.defaultfilters import truncatewords_html
 from django.utils.html import strip_tags
 from django.utils.translation import ugettext, ugettext_lazy as _
+from django.contrib.sites.managers import CurrentSiteManager
 
 from mezzanine.core.fields import RichTextField
 from mezzanine.core.managers import DisplayableManager
@@ -18,11 +20,15 @@ from mezzanine.utils.urls import slugify
 
 class Slugged(models.Model):
     """
-    Abstract model that handles auto-generating slugs.
+    Abstract model that handles auto-generating slugs. Each slugged
+    object is also affiliated with a specific site object.
     """
 
     title = models.CharField(_("Title"), max_length=100)
     slug = models.CharField(_("URL"), max_length=100, blank=True, null=True)
+    site = models.ForeignKey(Site, editable=False)
+
+    objects = CurrentSiteManager()
 
     class Meta:
         abstract = True
@@ -31,9 +37,11 @@ class Slugged(models.Model):
     def __unicode__(self):
         return self.title
 
-    def save(self, *args, **kwargs):
+    def save(self, update_site=False, *args, **kwargs):
         """
-        Create a unique slug by appending an index.
+        Create a unique slug by appending an index. Set the site to
+        the current site when the record is first created, unless the
+        ``update_site`` argument is explicitly set to ``True``.
         """
         if not self.slug:
             # For custom content types, use the ``Page`` instance for
@@ -54,6 +62,8 @@ class Slugged(models.Model):
                 except ObjectDoesNotExist:
                     break
                 i += 1
+        if update_site or not self.id:
+            self.site = Site.objects.get_current()
         super(Slugged, self).save(*args, **kwargs)
 
     def natural_key(self):
@@ -102,7 +112,6 @@ class Displayable(Slugged, MetaData):
         help_text=_("With published checked, won't be shown after this time"),
         blank=True, null=True)
     short_url = models.URLField(blank=True, null=True)
-    site = models.ForeignKey(Site, editable=False)
 
     objects = DisplayableManager()
     search_fields = {"keywords": 10, "title": 5}
@@ -110,11 +119,10 @@ class Displayable(Slugged, MetaData):
     class Meta:
         abstract = True
 
-    def save(self, update_site=True, *args, **kwargs):
+    def save(self, *args, **kwargs):
         """
         Set default for ``publsh_date`` and ``description`` if none
-        given. Unless the ``update_site`` argument is ``False``, set
-        the site to the current site.
+        given.
         """
         if self.publish_date is None:
             # publish_date will be blank when a blog post is created
@@ -122,8 +130,6 @@ class Displayable(Slugged, MetaData):
             self.publish_date = datetime.now()
         if not self.description:
             self.description = strip_tags(self.description_from_content())
-        if update_site:
-            self.site = Site.objects.get_current()
         super(Displayable, self).save(*args, **kwargs)
 
     def description_from_content(self):
@@ -200,9 +206,9 @@ class Orderable(models.Model):
     """
     Abstract model that provides a custom ordering integer field
     similar to using Meta's ``order_with_respect_to``, since to
-    date (Django 1.2) this doesn't work with ``ForeignKey("self")``.
-    We may also want this feature for models that aren't ordered with
-    respect to a particular field.
+    date (Django 1.2) this doesn't work with ``ForeignKey("self")``,
+    or with Generic Relations. We may also want this feature for
+    models that aren't ordered with respect to a particular field.
     """
 
     __metaclass__ = OrderableBase
@@ -216,13 +222,22 @@ class Orderable(models.Model):
         """
         Returns a dict to use as a filter for ordering operations
         containing the original ``Meta.order_with_respect_to`` value
-        if provided.
+        if provided. If the field is a Generic Relation, the dict
+        returned contains names and values for looking up the
+        relation's ``ct_field`` and ``fk_field`` attributes.
         """
         try:
-            field = self.order_with_respect_to
-            return {field: getattr(self, field)}
+            name = self.order_with_respect_to
+            value = getattr(self, name)
         except AttributeError:
+            # No ``order_with_respect_to`` specified on the model.
             return {}
+        # Support for generic relations.
+        field = getattr(self.__class__, name)
+        if isinstance(field, GenericForeignKey):
+            names = (field.ct_field, field.fk_field)
+            return dict([(name, getattr(self, name)) for name in names])
+        return {name: value}
 
     def save(self, *args, **kwargs):
         """
@@ -230,6 +245,7 @@ class Orderable(models.Model):
         """
         if self._order is None:
             lookup = self.with_respect_to()
+            lookup["_order__isnull"] = False
             concrete_model = base_concrete_model(Orderable, self)
             self._order = concrete_model.objects.filter(**lookup).count()
         super(Orderable, self).save(*args, **kwargs)
